@@ -9,7 +9,7 @@ use DynaLoader();
 use Carp ();
 @ISA = qw(DynaLoader);
 
-$VERSION = '4.00';
+$VERSION = '4.018_01';
 
 bootstrap DBD::mysql $VERSION;
 
@@ -98,6 +98,7 @@ sub AUTOLOAD {
 package DBD::mysql::dr; # ====== DRIVER ======
 use strict;
 use DBI qw(:sql_types);
+use DBI::Const::GetInfoType;
 
 sub connect {
     my($drh, $dsn, $username, $password, $attrhash) = @_;
@@ -109,6 +110,7 @@ sub connect {
     # Avoid warnings for undefined values
     $username ||= '';
     $password ||= '';
+    $attrhash ||= {};
 
     # create a 'blank' dbh
     my($this, $privateAttrHash) = (undef, $attrhash);
@@ -251,38 +253,134 @@ sub _SelectDB ($$) {
     die "_SelectDB is removed from this module; use DBI->connect instead.";
 }
 
-{
-    my $names = ['TABLE_CAT', 'TABLE_SCHEM', 'TABLE_NAME',
-		 'TABLE_TYPE', 'REMARKS'];
+sub table_info ($) {
+  my ($dbh, $catalog, $schema, $table, $type, $attr) = @_;
+  $dbh->{mysql_server_prepare}||= 0;
+  my $mysql_server_prepare_save= $dbh->{mysql_server_prepare};
+  $dbh->{mysql_server_prepare}= 0;
+  my @names = qw(TABLE_CAT TABLE_SCHEM TABLE_NAME TABLE_TYPE REMARKS);
+  my @rows;
 
-    sub table_info ($) {
-	my $dbh = shift;
-	my $sth = $dbh->prepare("SHOW TABLES");
-	return undef unless $sth;
-	if (!$sth->execute()) {
-	  return DBI::set_err($dbh, $sth->err(), $sth->errstr());
-        }
-	my @tables;
-	while (my $ref = $sth->fetchrow_arrayref()) {
-	  push(@tables, [ undef, undef, $ref->[0], 'TABLE', undef ]);
-        }
-	my $dbh2;
-	if (!($dbh2 = $dbh->{'~dbd_driver~_sponge_dbh'})) {
-	    $dbh2 = $dbh->{'~dbd_driver~_sponge_dbh'} =
-		DBI->connect("DBI:Sponge:");
-	    if (!$dbh2) {
-	        DBI::set_err($dbh, 1, $DBI::errstr);
-		return undef;
-	    }
-	}
-	my $sth2 = $dbh2->prepare("SHOW TABLES", { 'rows' => \@tables,
-						   'NAME' => $names,
-						   'NUM_OF_FIELDS' => 5 });
-	if (!$sth2) {
-	    DBI::set_err($sth2, $dbh2->err(), $dbh2->errstr());
-	}
-	$sth2;
+  my $sponge = DBI->connect("DBI:Sponge:", '','')
+    or return $dbh->DBI::set_err($DBI::err, "DBI::Sponge: $DBI::errstr");
+
+# Return the list of catalogs
+  if (defined $catalog && $catalog eq "%" &&
+      (!defined($schema) || $schema eq "") &&
+      (!defined($table) || $table eq ""))
+  {
+    @rows = (); # Empty, because MySQL doesn't support catalogs (yet)
+  }
+  # Return the list of schemas
+  elsif (defined $schema && $schema eq "%" &&
+      (!defined($catalog) || $catalog eq "") &&
+      (!defined($table) || $table eq ""))
+  {
+    my $sth = $dbh->prepare("SHOW DATABASES")
+      or ($dbh->{mysql_server_prepare}= $mysql_server_prepare_save && 
+          return undef);
+
+    $sth->execute()
+      or ($dbh->{mysql_server_prepare}= $mysql_server_prepare_save && 
+        return DBI::set_err($dbh, $sth->err(), $sth->errstr()));
+
+    while (my $ref = $sth->fetchrow_arrayref())
+    {
+      push(@rows, [ undef, $ref->[0], undef, undef, undef ]);
     }
+  }
+  # Return the list of table types
+  elsif (defined $type && $type eq "%" &&
+      (!defined($catalog) || $catalog eq "") &&
+      (!defined($schema) || $schema eq "") &&
+      (!defined($table) || $table eq ""))
+  {
+    @rows = (
+        [ undef, undef, undef, "TABLE", undef ],
+        [ undef, undef, undef, "VIEW",  undef ],
+        );
+  }
+  # Special case: a catalog other than undef, "", or "%"
+  elsif (defined $catalog && $catalog ne "" && $catalog ne "%")
+  {
+    @rows = (); # Nothing, because MySQL doesn't support catalogs yet.
+  }
+  # Uh oh, we actually have a meaty table_info call. Work is required!
+  else
+  {
+    my @schemas;
+    # If no table was specified, we want them all
+    $table ||= "%";
+
+    # If something was given for the schema, we need to expand it to
+    # a list of schemas, since it may be a wildcard.
+    if (defined $schema && $schema ne "")
+    {
+      my $sth = $dbh->prepare("SHOW DATABASES LIKE " .
+          $dbh->quote($schema))
+        or ($dbh->{mysql_server_prepare}= $mysql_server_prepare_save && 
+        return undef);
+      $sth->execute()
+        or ($dbh->{mysql_server_prepare}= $mysql_server_prepare_save && 
+        return DBI::set_err($dbh, $sth->err(), $sth->errstr()));
+
+      while (my $ref = $sth->fetchrow_arrayref())
+      {
+        push @schemas, $ref->[0];
+      }
+    }
+    # Otherwise we want the current database
+    else
+    {
+      push @schemas, $dbh->selectrow_array("SELECT DATABASE()");
+    }
+
+    # Figure out which table types are desired
+    my ($want_tables, $want_views);
+    if (defined $type && $type ne "")
+    {
+      $want_tables = ($type =~ m/table/i);
+      $want_views  = ($type =~ m/view/i);
+    }
+    else
+    {
+      $want_tables = $want_views = 1;
+    }
+
+    for my $database (@schemas)
+    {
+      my $sth = $dbh->prepare("SHOW /*!50002 FULL*/ TABLES FROM " .
+          $dbh->quote_identifier($database) .
+          " LIKE " .  $dbh->quote($table))
+          or ($dbh->{mysql_server_prepare}= $mysql_server_prepare_save && 
+          return undef);
+
+      $sth->execute() or
+          ($dbh->{mysql_server_prepare}= $mysql_server_prepare_save &&
+          return DBI::set_err($dbh, $sth->err(), $sth->errstr()));
+
+      while (my $ref = $sth->fetchrow_arrayref())
+      {
+        my $type = (defined $ref->[1] &&
+            $ref->[1] =~ /view/i) ? 'VIEW' : 'TABLE';
+        next if $type eq 'TABLE' && not $want_tables;
+        next if $type eq 'VIEW'  && not $want_views;
+        push @rows, [ undef, $database, $ref->[0], $type, undef ];
+      }
+    }
+  }
+
+  my $sth = $sponge->prepare("table_info",
+  {
+    rows          => \@rows,
+    NUM_OF_FIELDS => scalar @names,
+    NAME          => \@names,
+  }) 
+    or ($dbh->{mysql_server_prepare}= $mysql_server_prepare_save && 
+      return $dbh->DBI::set_err($sponge->err(), $sponge->errstr()));
+
+  $dbh->{mysql_server_prepare}= $mysql_server_prepare_save;
+  return $sth;
 }
 
 sub _ListTables {
@@ -295,145 +393,332 @@ sub _ListTables {
 
 
 sub column_info {
-    my ($dbh, $catalog, $schema, $table, $column) = @_;
-    return $dbh->set_err(1, "column_info doesn't support table wildcard")
-	if $table !~ /^\w+$/;
-    return $dbh->set_err(1, "column_info doesn't support column selection")
-	if $column ne "%";
+  my ($dbh, $catalog, $schema, $table, $column) = @_;
+  $dbh->{mysql_server_prepare}||= 0;
+  my $mysql_server_prepare_save= $dbh->{mysql_server_prepare};
+  $dbh->{mysql_server_prepare}= 0;
 
-    my $table_id = $dbh->quote_identifier($catalog, $schema, $table);
+  # ODBC allows a NULL to mean all columns, so we'll accept undef
+  $column = '%' unless defined $column;
 
-    my @names = qw(
-	TABLE_CAT TABLE_SCHEM TABLE_NAME COLUMN_NAME
-	DATA_TYPE TYPE_NAME COLUMN_SIZE BUFFER_LENGTH DECIMAL_DIGITS
-	NUM_PREC_RADIX NULLABLE REMARKS COLUMN_DEF
-	SQL_DATA_TYPE SQL_DATETIME_SUB CHAR_OCTET_LENGTH
-	ORDINAL_POSITION IS_NULLABLE CHAR_SET_CAT
-	CHAR_SET_SCHEM CHAR_SET_NAME COLLATION_CAT COLLATION_SCHEM COLLATION_NAME
-	UDT_CAT UDT_SCHEM UDT_NAME DOMAIN_CAT DOMAIN_SCHEM DOMAIN_NAME
-	SCOPE_CAT SCOPE_SCHEM SCOPE_NAME MAX_CARDINALITY
-	DTD_IDENTIFIER IS_SELF_REF
-	mysql_is_pri_key mysql_type_name mysql_values
-    );
-    my %col_info;
+  my $ER_NO_SUCH_TABLE= 1146;
 
-    local $dbh->{FetchHashKeyName} = 'NAME_lc';
-    my $desc_sth = $dbh->prepare("DESCRIBE $table_id");
-    my $desc = $dbh->selectall_arrayref($desc_sth, { Columns=>{} });
-    my $ordinal_pos = 0;
-    foreach my $row (@$desc) {
-	my $type = $row->{type};
-	$type =~ m/^(\w+)(?:\((.*?)\))?\s*(.*)/;
-	my $basetype = lc($1);
+  my $table_id = $dbh->quote_identifier($catalog, $schema, $table);
 
-	my $info = $col_info{ $row->{field} } = {
-	    TABLE_CAT   => $catalog,
-	    TABLE_SCHEM => $schema,
-	    TABLE_NAME  => $table,
-	    COLUMN_NAME => $row->{field},
-	    NULLABLE    => ($row->{null} eq 'YES') ? 1 : 0,
-	    IS_NULLABLE => ($row->{null} eq 'YES') ? "YES" : "NO",
-	    TYPE_NAME   => uc($basetype),
-	    COLUMN_DEF  => $row->{default},
-	    ORDINAL_POSITION => ++$ordinal_pos,
-	    mysql_is_pri_key => ($row->{key}  eq 'PRI'),
-	    mysql_type_name  => $row->{type},
-	};
-	# This code won't deal with a pathalogical case where a value
-	# contains a single quote followed by a comma, and doesn't unescape
-	# any escaped values. But who would use those in an enum or set?
-	my @type_params = ($2 && index($2,"'")>=0)
-			? ("$2," =~ /'(.*?)',/g)  # assume all are quoted
-			: split /,/, $2||'';      # no quotes, plain list
-	s/''/'/g for @type_params;                # undo doubling of quotes
-	my @type_attr = split / /, $3||'';
-	#warn "$type: $basetype [@type_params] [@type_attr]\n";
+  my @names = qw(
+      TABLE_CAT TABLE_SCHEM TABLE_NAME COLUMN_NAME
+      DATA_TYPE TYPE_NAME COLUMN_SIZE BUFFER_LENGTH DECIMAL_DIGITS
+      NUM_PREC_RADIX NULLABLE REMARKS COLUMN_DEF
+      SQL_DATA_TYPE SQL_DATETIME_SUB CHAR_OCTET_LENGTH
+      ORDINAL_POSITION IS_NULLABLE CHAR_SET_CAT
+      CHAR_SET_SCHEM CHAR_SET_NAME COLLATION_CAT COLLATION_SCHEM COLLATION_NAME
+      UDT_CAT UDT_SCHEM UDT_NAME DOMAIN_CAT DOMAIN_SCHEM DOMAIN_NAME
+      SCOPE_CAT SCOPE_SCHEM SCOPE_NAME MAX_CARDINALITY
+      DTD_IDENTIFIER IS_SELF_REF
+      mysql_is_pri_key mysql_type_name mysql_values
+      mysql_is_auto_increment
+      );
+  my %col_info;
 
-	$info->{DATA_TYPE} = SQL_VARCHAR();
-	if ($basetype =~ /^(char|varchar|\w*text|\w*blob)/) {
-	    $info->{DATA_TYPE} = SQL_CHAR() if $basetype eq 'char';
-	    if ($type_params[0]) {
-		$info->{COLUMN_SIZE} = $type_params[0];
-	    }
-	    else {
-		$info->{COLUMN_SIZE} = 65535;
-		$info->{COLUMN_SIZE} = 255        if $basetype =~ /^tiny/;
-		$info->{COLUMN_SIZE} = 16777215   if $basetype =~ /^medium/;
-		$info->{COLUMN_SIZE} = 4294967295 if $basetype =~ /^long/;
-	    }
-	}
-	elsif ($basetype =~ /^(binary|varbinary)/) {
-	    $info->{COLUMN_SIZE} = $type_params[0];
+  local $dbh->{FetchHashKeyName} = 'NAME_lc';
+  # only ignore ER_NO_SUCH_TABLE in internal_execute if issued from here
+  my $desc_sth = $dbh->prepare("DESCRIBE $table_id " . $dbh->quote($column));
+  my $desc = $dbh->selectall_arrayref($desc_sth, { Columns=>{} });
+
+  #return $desc_sth if $desc_sth->err();
+  if (my $err = $desc_sth->err())
+  {
+    # return the error, unless it is due to the table not 
+    # existing per DBI spec
+    if ($err != $ER_NO_SUCH_TABLE)
+    {
+      $dbh->{mysql_server_prepare}= $mysql_server_prepare_save;
+      return undef;
+    }
+    $dbh->set_err(undef,undef);
+    $desc = [];
+  }
+
+  my $ordinal_pos = 0;
+  for my $row (@$desc)
+  {
+    my $type = $row->{type};
+    $type =~ m/^(\w+)(?:\((.*?)\))?\s*(.*)/;
+    my $basetype  = lc($1);
+    my $typemod   = $2;
+    my $attr      = $3;
+
+    my $info = $col_info{ $row->{field} }= {
+	    TABLE_CAT               => $catalog,
+	    TABLE_SCHEM             => $schema,
+	    TABLE_NAME              => $table,
+	    COLUMN_NAME             => $row->{field},
+	    NULLABLE                => ($row->{null} eq 'YES') ? 1 : 0,
+	    IS_NULLABLE             => ($row->{null} eq 'YES') ? "YES" : "NO",
+	    TYPE_NAME               => uc($basetype),
+	    COLUMN_DEF              => $row->{default},
+	    ORDINAL_POSITION        => ++$ordinal_pos,
+	    mysql_is_pri_key        => ($row->{key}  eq 'PRI'),
+	    mysql_type_name         => $row->{type},
+      mysql_is_auto_increment => ($row->{extra} =~ /auto_increment/i ? 1 : 0),
+    };
+    #
+	  # This code won't deal with a pathalogical case where a value
+	  # contains a single quote followed by a comma, and doesn't unescape
+	  # any escaped values. But who would use those in an enum or set?
+    #
+	  my @type_params= ($typemod && index($typemod,"'")>=0) ?
+      ("$typemod," =~ /'(.*?)',/g)  # assume all are quoted
+			: split /,/, $typemod||'';      # no quotes, plain list
+	  s/''/'/g for @type_params;                # undo doubling of quotes
+
+	  my @type_attr= split / /, $attr||'';
+
+  	$info->{DATA_TYPE}= SQL_VARCHAR();
+    if ($basetype =~ /^(char|varchar|\w*text|\w*blob)/)
+    {
+      $info->{DATA_TYPE}= SQL_CHAR() if $basetype eq 'char';
+      if ($type_params[0])
+      {
+        $info->{COLUMN_SIZE} = $type_params[0];
+      }
+      else
+      {
+        $info->{COLUMN_SIZE} = 65535;
+        $info->{COLUMN_SIZE} = 255        if $basetype =~ /^tiny/;
+        $info->{COLUMN_SIZE} = 16777215   if $basetype =~ /^medium/;
+        $info->{COLUMN_SIZE} = 4294967295 if $basetype =~ /^long/;
+      }
+    }
+	  elsif ($basetype =~ /^(binary|varbinary)/)
+    {
+      $info->{COLUMN_SIZE} = $type_params[0];
 	    # SQL_BINARY & SQL_VARBINARY are tempting here but don't match the
 	    # semantics for mysql (not hex). SQL_CHAR &  SQL_VARCHAR are correct here.
 	    $info->{DATA_TYPE} = ($basetype eq 'binary') ? SQL_CHAR() : SQL_VARCHAR();
-	}
-	elsif ($basetype =~ /^(enum|set)/) {
-	    if ($basetype eq 'set') {
-		$info->{COLUMN_SIZE} = length(join ",", @type_params);
+    }
+    elsif ($basetype =~ /^(enum|set)/)
+    {
+	    if ($basetype eq 'set')
+      {
+		    $info->{COLUMN_SIZE} = length(join ",", @type_params);
 	    }
-	    else {
-		my $max_len = 0;
-		length($_) > $max_len and $max_len = length($_) for @type_params;
-		$info->{COLUMN_SIZE} = $max_len;
+	    else
+      {
+        my $max_len = 0;
+        length($_) > $max_len and $max_len = length($_) for @type_params;
+        $info->{COLUMN_SIZE} = $max_len;
 	    }
 	    $info->{"mysql_values"} = \@type_params;
-	}
-	elsif ($basetype =~ /int/) { # big/medium/small/tiny etc + unsigned?
+    }
+    elsif ($basetype =~ /int/)
+    { 
+      # big/medium/small/tiny etc + unsigned?
 	    $info->{DATA_TYPE} = SQL_INTEGER();
 	    $info->{NUM_PREC_RADIX} = 10;
 	    $info->{COLUMN_SIZE} = $type_params[0];
-	}
-	elsif ($basetype =~ /^decimal/) {
-	    $info->{DATA_TYPE} = SQL_DECIMAL();
-	    $info->{NUM_PREC_RADIX} = 10;
-	    $info->{COLUMN_SIZE}    = $type_params[0];
-	    $info->{DECIMAL_DIGITS} = $type_params[1];
-	}
-	elsif ($basetype =~ /^(float|double)/) {
+    }
+    elsif ($basetype =~ /^decimal/)
+    {
+      $info->{DATA_TYPE} = SQL_DECIMAL();
+      $info->{NUM_PREC_RADIX} = 10;
+      $info->{COLUMN_SIZE}    = $type_params[0];
+      $info->{DECIMAL_DIGITS} = $type_params[1];
+    }
+    elsif ($basetype =~ /^(float|double)/)
+    {
 	    $info->{DATA_TYPE} = ($basetype eq 'float') ? SQL_FLOAT() : SQL_DOUBLE();
 	    $info->{NUM_PREC_RADIX} = 2;
 	    $info->{COLUMN_SIZE} = ($basetype eq 'float') ? 32 : 64;
-	}
-	elsif ($basetype =~ /date|time/) { # date/datetime/time/timestamp
-	    if ($basetype eq 'time' or $basetype eq 'date') {
-		#$info->{DATA_TYPE}   = ($basetype eq 'time') ? SQL_TYPE_TIME() : SQL_TYPE_DATE();
-                $info->{DATA_TYPE}   = ($basetype eq 'time') ? SQL_TIME() : SQL_DATE(); 
-		$info->{COLUMN_SIZE} = ($basetype eq 'time') ? 8 : 10;
+    }
+    elsif ($basetype =~ /date|time/)
+    { 
+      # date/datetime/time/timestamp
+	    if ($basetype eq 'time' or $basetype eq 'date')
+      {
+		    #$info->{DATA_TYPE}   = ($basetype eq 'time') ? SQL_TYPE_TIME() : SQL_TYPE_DATE();
+        $info->{DATA_TYPE}   = ($basetype eq 'time') ? SQL_TIME() : SQL_DATE(); 
+        $info->{COLUMN_SIZE} = ($basetype eq 'time') ? 8 : 10;
+      }
+	    else
+      {
+        # datetime/timestamp
+        #$info->{DATA_TYPE}     = SQL_TYPE_TIMESTAMP();
+		    $info->{DATA_TYPE}        = SQL_TIMESTAMP();
+		    $info->{SQL_DATA_TYPE}    = SQL_DATETIME();
+        $info->{SQL_DATETIME_SUB} = $info->{DATA_TYPE} - ($info->{SQL_DATA_TYPE} * 10);
+        $info->{COLUMN_SIZE}      = ($basetype eq 'datetime') ? 19 : $type_params[0] || 14;
 	    }
-	    else { # datetime/timestamp
-		#$info->{DATA_TYPE}     = SQL_TYPE_TIMESTAMP();
-		$info->{DATA_TYPE}     = SQL_TIMESTAMP();
-		$info->{SQL_DATA_TYPE} = SQL_DATETIME();
-	        $info->{SQL_DATETIME_SUB} = $info->{DATA_TYPE} - ($info->{SQL_DATA_TYPE} * 10);
-		$info->{COLUMN_SIZE}   = ($basetype eq 'datetime') ? 19 : $type_params[0] || 14;
-	    }
-	    $info->{DECIMAL_DIGITS} = 0; # no fractional seconds
-	}
-	elsif ($basetype eq 'year') {	# no close standard so treat as int
-	    $info->{DATA_TYPE} = SQL_INTEGER();
+	    $info->{DECIMAL_DIGITS}= 0; # no fractional seconds
+    }
+    elsif ($basetype eq 'year')
+    {	
+      # no close standard so treat as int
+	    $info->{DATA_TYPE}      = SQL_INTEGER();
 	    $info->{NUM_PREC_RADIX} = 10;
-	    $info->{COLUMN_SIZE} = 4;
-	}
-	else {
+	    $info->{COLUMN_SIZE}    = 4;
+	  }
+	  else
+    {
 	    Carp::carp("column_info: unrecognized column type '$basetype' of $table_id.$row->{field} treated as varchar");
-	}
-	$info->{SQL_DATA_TYPE} ||= $info->{DATA_TYPE};
-	#warn Dumper($info);
+    }
+    $info->{SQL_DATA_TYPE} ||= $info->{DATA_TYPE};
+    #warn Dumper($info);
+  }
+
+  my $sponge = DBI->connect("DBI:Sponge:", '','')
+    or (  $dbh->{mysql_server_prepare}= $mysql_server_prepare_save &&
+          return $dbh->DBI::set_err($DBI::err, "DBI::Sponge: $DBI::errstr"));
+
+  my $sth = $sponge->prepare("column_info $table", {
+      rows          => [ map { [ @{$_}{@names} ] } values %col_info ],
+      NUM_OF_FIELDS => scalar @names,
+      NAME          => \@names,
+      }) or
+  return ($dbh->{mysql_server_prepare}= $mysql_server_prepare_save &&
+          $dbh->DBI::set_err($sponge->err(), $sponge->errstr()));
+
+  $dbh->{mysql_server_prepare}= $mysql_server_prepare_save;
+  return $sth;
+}
+
+
+sub primary_key_info {
+  my ($dbh, $catalog, $schema, $table) = @_;
+  $dbh->{mysql_server_prepare}||= 0;
+  my $mysql_server_prepare_save= $dbh->{mysql_server_prepare};
+
+  my $table_id = $dbh->quote_identifier($catalog, $schema, $table);
+
+  my @names = qw(
+      TABLE_CAT TABLE_SCHEM TABLE_NAME COLUMN_NAME KEY_SEQ PK_NAME
+      );
+  my %col_info;
+
+  local $dbh->{FetchHashKeyName} = 'NAME_lc';
+  my $desc_sth = $dbh->prepare("SHOW KEYS FROM $table_id");
+  my $desc= $dbh->selectall_arrayref($desc_sth, { Columns=>{} });
+  my $ordinal_pos = 0;
+  for my $row (grep { $_->{key_name} eq 'PRIMARY'} @$desc)
+  {
+    $col_info{ $row->{column_name} }= {
+      TABLE_CAT   => $catalog,
+      TABLE_SCHEM => $schema,
+      TABLE_NAME  => $table,
+      COLUMN_NAME => $row->{column_name},
+      KEY_SEQ     => $row->{seq_in_index},
+      PK_NAME     => $row->{key_name},
+    };
+  }
+
+  my $sponge = DBI->connect("DBI:Sponge:", '','')
+    or 
+     ($dbh->{mysql_server_prepare}= $mysql_server_prepare_save &&
+      return $dbh->DBI::set_err($DBI::err, "DBI::Sponge: $DBI::errstr"));
+
+  my $sth= $sponge->prepare("primary_key_info $table", {
+      rows          => [ map { [ @{$_}{@names} ] } values %col_info ],
+      NUM_OF_FIELDS => scalar @names,
+      NAME          => \@names,
+      }) or 
+       ($dbh->{mysql_server_prepare}= $mysql_server_prepare_save &&
+        return $dbh->DBI::set_err($sponge->err(), $sponge->errstr()));
+
+  $dbh->{mysql_server_prepare}= $mysql_server_prepare_save;
+
+  return $sth;
+}
+
+
+sub foreign_key_info {
+    my ($dbh,
+        $pk_catalog, $pk_schema, $pk_table,
+        $fk_catalog, $fk_schema, $fk_table,
+       ) = @_;
+
+    # INFORMATION_SCHEMA.KEY_COLUMN_USAGE was added in 5.0.6
+    my ($maj, $min, $point) = _version($dbh);
+    return if $maj < 5 || ($maj == 5 && $point < 6);
+
+    my $sql = <<'EOF';
+SELECT NULL AS PKTABLE_CAT,
+       A.REFERENCED_TABLE_SCHEMA AS PKTABLE_SCHEM,
+       A.REFERENCED_TABLE_NAME AS PKTABLE_NAME,
+       A.REFERENCED_COLUMN_NAME AS PKCOLUMN_NAME,
+       A.TABLE_CATALOG AS FKTABLE_CAT,
+       A.TABLE_SCHEMA AS FKTABLE_SCHEM,
+       A.TABLE_NAME AS FKTABLE_NAME,
+       A.COLUMN_NAME AS FKCOLUMN_NAME,
+       A.ORDINAL_POSITION AS KEY_SEQ,
+       NULL AS UPDATE_RULE,
+       NULL AS DELETE_RULE,
+       A.CONSTRAINT_NAME AS FK_NAME,
+       NULL AS PK_NAME,
+       NULL AS DEFERABILITY,
+       NULL AS UNIQUE_OR_PRIMARY
+  FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE A,
+       INFORMATION_SCHEMA.TABLE_CONSTRAINTS B
+ WHERE A.TABLE_SCHEMA = B.TABLE_SCHEMA AND A.TABLE_NAME = B.TABLE_NAME
+   AND A.CONSTRAINT_NAME = B.CONSTRAINT_NAME AND B.CONSTRAINT_TYPE IS NOT NULL
+EOF
+
+    my @where;
+    my @bind;
+
+    # catalogs are not yet supported by MySQL
+
+#    if (defined $pk_catalog) {
+#        push @where, 'A.REFERENCED_TABLE_CATALOG = ?';
+#        push @bind, $pk_catalog;
+#    }
+
+    if (defined $pk_schema) {
+        push @where, 'A.REFERENCED_TABLE_SCHEMA = ?';
+        push @bind, $pk_schema;
     }
 
-    my $sponge = DBI->connect("DBI:Sponge:", '','')
-	or return $dbh->DBI::set_err($DBI::err, "DBI::Sponge: $DBI::errstr");
-    my $sth = $sponge->prepare("column_info $table", {
-	rows => [ map { [ @{$_}{@names} ] } values %col_info ],
-	NUM_OF_FIELDS => scalar @names,
-	NAME => \@names,
-    }) or return $dbh->DBI::set_err($sponge->err(), $sponge->errstr());
+    if (defined $pk_table) {
+        push @where, 'A.REFERENCED_TABLE_NAME = ?';
+        push @bind, $pk_table;
+    }
+
+#    if (defined $fk_catalog) {
+#        push @where, 'A.TABLE_CATALOG = ?';
+#        push @bind,  $fk_schema;
+#    }
+
+    if (defined $fk_schema) {
+        push @where, 'A.TABLE_SCHEMA = ?';
+        push @bind,  $fk_schema;
+    }
+
+    if (defined $fk_table) {
+        push @where, 'A.TABLE_NAME = ?';
+        push @bind,  $fk_table;
+    }
+
+    if (@where) {
+        $sql .= ' AND ';
+        $sql .= join ' AND ', @where;
+    }
+    $sql .= " ORDER BY A.TABLE_SCHEMA, A.TABLE_NAME, A.ORDINAL_POSITION";
+
+    local $dbh->{FetchHashKeyName} = 'NAME_uc';
+    my $sth = $dbh->prepare($sql);
+    $sth->execute(@bind);
 
     return $sth;
 }
 
 
+sub _version {
+    my $dbh = shift;
+
+    return
+        $dbh->get_info($DBI::Const::GetInfoType::GetInfoType{SQL_DBMS_VER})
+            =~ /(\d+)\.(\d+)\.(\d+)/;
+}
+
 
 ####################
 # get_info()
@@ -661,6 +946,12 @@ If your DSN contains the option "mysql_connect_timeout=##", the connect
 request to the server will timeout if it has not been successful after
 the given number of seconds.
 
+ =item mysql_init_command
+
+ If your DSN contains the option "mysql_init_command_timeout=##", then
+ this SQL statement is executed when connecting to the MySQL server.
+ It is automatically re-executed if reconnection occurs.
+
 =item mysql_read_default_file
 
 =item mysql_read_default_group
@@ -743,6 +1034,12 @@ in the MySQL client library by default. If your DSN contains the option
 this option is *ineffective* if the server has also been configured to
 disallow LOCAL.)
 
+=item mysql_multi_statements
+
+As of MySQL 4.1, support for multiple statements seperated by a semicolon
+(;) may be enabled by using this option. Enabling this option may cause
+problems if server-side prepared statements are also enabled.
+
 =item Prepared statement support (server side prepare)
 
 As of 3.0002_1, server side prepare statements were on by default (if your
@@ -780,7 +1077,12 @@ options to embedded server.
 
 Example:
 
+use DBI;
 $testdsn="DBI:mysqlEmb:database=test;mysql_embedded_options=--help,--verbose";
+$dbh = DBI->connect($testdsn,"a","b");
+
+This would cause the command line help to the embedded MySQL server library
+to be printed.
 
 
 =item mysql_embedded_groups
@@ -980,6 +1282,103 @@ need to issue the command C<SET NAMES utf8> to get the same effect.
 
 This option is experimental and may change in future versions.
 
+=item mysql_bind_type_guessing
+
+This attribute causes the driver (emulated prepare statements) 
+to attempt to guess if a value being bound is a numeric value,
+and if so, doesn't quote the value.  This was created by 
+Dragonchild and is one way to deal with the performance issue 
+of using quotes in a statement that is inserting or updating a
+large numeric value. This was previously called 
+C<unsafe_bind_type_guessing> because it is experimental. I have 
+successfully run the full test suite with this option turned on,
+the name can now be simply C<mysql_bind_type_guessing>. 
+
+CAVEAT: Even though you can insert an integer value into a 
+character column, if this column is indexed, if you query that
+column with the integer value not being quoted, it will not 
+use the index:
+
+MariaDB [test]> explain select * from test where value0 = '3' \G
+*************************** 1. row ***************************
+           id: 1
+  select_type: SIMPLE
+        table: test
+         type: ref
+possible_keys: value0
+          key: value0
+      key_len: 13
+          ref: const
+         rows: 1
+        Extra: Using index condition
+1 row in set (0.00 sec)
+
+MariaDB [test]> explain select * from test where value0 = 3
+    -> \G
+*************************** 1. row ***************************
+           id: 1
+  select_type: SIMPLE
+        table: test
+         type: ALL
+possible_keys: value0
+          key: NULL
+      key_len: NULL
+          ref: NULL
+         rows: 6
+        Extra: Using where
+1 row in set (0.00 sec)
+
+
+See bug: https://rt.cpan.org/Ticket/Display.html?id=43822
+
+=item mysql_bind_comment_placeholders
+
+This attribute causes the driver (emulated prepare statements) 
+will cause any placeholders in comments to be bound. This is 
+not correct prepared statement behavior, but some developers
+have come to depend on this behavior, so I have made it available 
+in 4.015
+
+See bug: https://rt.cpan.org/Ticket/Display.html?id=
+
+C<mysql_bind_type_guessing> can be turned on via 
+
+ - through DSN 
+
+  my $dbh= DBI->connect('DBI:mysql:test', 'username', 'pass',
+  { mysql_bind_type_guessing => 1})
+
+  - OR after handle creation
+
+  $dbh->{mysql_bind_type_guessing} = 1;
+
+=item mysql_no_autocommit_cmd
+
+This attribute causes the driver to not issue 'set autocommit'
+either through explicit or using mysql_autocommit(). This is 
+particularly useful in the case of using MySQL Proxy.
+
+See the bug report:
+
+https://rt.cpan.org/Public/Bug/Display.html?id=46308
+
+As well as:
+
+http://bugs.mysql.com/bug.php?id=32464
+
+C<mysql_no_autocommit_cmd> can be turned on via 
+
+ - through DSN 
+
+  my $dbh= DBI->connect('DBI:mysql:test', 'username', 'pass',
+  { mysql_no_autocommit_cmd => 1})
+
+  - OR after handle creation
+
+  $dbh->{mysql_no_autocommit_cmd} = 1;
+
+
+
 =head1 STATEMENT HANDLES
 
 The statement handles of DBD::mysql support a number
@@ -1083,6 +1482,27 @@ the maximum physically present in the result table, I<length> gives
 the theoretically possible maximum. I<max_length> is valid for MySQL
 only.
 
+=item mysql_clientinfo
+
+List information of the MySQL client library that DBD::mysql was built
+against:
+
+print "$dbh->{mysql_clientinfo}\n";
+
+5.2.0-MariaDB
+
+=item mysql_clientversion
+
+print "$dbh->{mysql_clientversion}\n";
+
+50200
+
+=item mysql_serverversion
+
+print "$dbh->{mysql_serverversion}\n";
+
+50200
+
 =item NAME
 
 A reference to an array of column names.
@@ -1124,8 +1544,11 @@ DBI::SQL_SMALLINT() or DBI::SQL_VARCHAR().
 Similar to mysql, but type names and not numbers are returned.
 Whenever possible, the ANSI SQL name is preferred.
 
-=back
+=item mysql_warning_count
 
+The number of warnings generated during execution of the SQL statement.
+
+=back
 
 =head1 TRANSACTION SUPPORT
 
@@ -1506,24 +1929,50 @@ in the PPM program.
 
 =head1 AUTHORS
 
-The current version of B<DBD::mysql> is almost completely written
-by Jochen Wiedmann, and is now being maintained by
-Patrick Galbraith (I<patg@mysql.com>). 
-The first version's author was Alligator Descartes, who was aided
-and abetted by Gary Shea, Andreas Knig and Tim Bunce amongst others.
+Originally, there was a non-DBI driver, Mysql, which was much like 
+PHP drivers such as mysql and mysqli. The B<Mysql> module was 
+originally written by Andreas Knig <koenig@kulturbox.de> who still, to this
+day, contributes patches to DBD::mysql. An emulated version of Mysql was
+provided to DBD::mysql from Jochen Wiedmann, but eventually deprecated as it
+was another bundle of code to maintain.
 
-The B<Mysql> module was originally written by Andreas Knig
-<koenig@kulturbox.de>. The current version, mainly an emulation
-layer, is from Jochen Wiedmann.
+The first incarnation of DBD::mysql was developed by Alligator Descartes,
+who was also aided and abetted by Gary Shea, Andreas Knig and 
+Tim Bunce.
+
+The current incarnation of B<DBD::mysql> was written by Jochen Wiedmann,
+then numerous changes and bug-fixes were added by Rudy Lippan. Next, 
+prepared statement support was added by Patrick Galbraith and 
+Alexy Stroganov (who also soley added embedded server 
+support).
+
+For the past seven years DBD::mysql has been maintained by
+Patrick Galbraith (I<patg@patg.net>) along with the entire community 
+of Perl developers who keep sending patches and making Patrick's job
+easier. 
+
+
+=head1 CONTRIBUTIONS
+
+Anyone who desires to contribute to this project is encouraged to do so.
+Currently, the sourcecode for this project can be found at Github: 
+
+git://github.com/CaptTofu/DBD-mysql.git
+
+Either fork this repository and produce a branch with your changeset that 
+the maintainer can merge to his tree, or create a diff with git. The maintainer
+is more than glad to take contributions from the community as
+many features and fixes from DBD::mysql have come from the community.
 
 
 =head1 COPYRIGHT
 
 
 This module is 
-Large Portions Copyright (c) 2004-2006 MySQL Patrick Galbraith, Alexey Stroganov,
-Large Portions Copyright (c) 2003-2005 Rudolf Lippan; Large Portions 
-Copyright (c) 1997-2003 Jochen Wiedmann, with code portions 
+Large Portions Copyright (c) 2004-2010 Patrick Galbraith
+Large Portions Copyright (c) 2004-2006 Alexey Stroganov
+Large Portions Copyright (c) 2003-2005 Rudolf Lippan
+Large Portions Copyright (c) 1997-2003 Jochen Wiedmann, with code portions 
 Copyright (c)1994-1997 their original authors This module is
 released under the same license as Perl itself. See the Perl README
 for details.
@@ -1556,6 +2005,10 @@ Also, the main DBI site is at
 
 http://dbi.perl.org/
 
+And source:
+
+git://github.com/CaptTofu/DBD-mysql.git
+
 =head1 ADDITIONAL DBI INFORMATION
 
 Additional information on the DBI project can be found on the World
